module Tins
  require 'tins/xt/string_camelize'
  require 'tins/xt/string_underscore'
  require 'tins/xt/string_version'
  require 'tins/xt/string_byte_order_mark'
  require 'tins/xt/string_named_placeholders'
end
